import pytest

from program.tests.factories import FundingCategoryFactory

pytestmark = pytest.mark.django_db


def url(funding_category=None):
    return (
        f"/api/v1/funding-categories/{funding_category.id}/"
        if funding_category
        else "/api/v1/funding-categories/"
    )


def test_create_funding_category(admin_api_client):
    data = {
        "name": "NAME",
        "is_active": True,
        "slug": "funding-category",
    }

    response = admin_api_client.post(url(), data, format="json")

    assert response.status_code == 201

    assert response.data["is_active"] == data["is_active"]
    assert response.data["name"] == data["name"]
    assert response.data["slug"] == data["slug"]


def test_retrieve_funding_category(common_api_client1, funding_category):
    response = common_api_client1.get(url(funding_category=funding_category))

    assert response.status_code == 200

    assert response.data["is_active"] == funding_category.is_active
    assert response.data["name"] == funding_category.name
    assert response.data["slug"] == funding_category.slug


def test_delete_funding_category(admin_api_client, funding_category):
    response = admin_api_client.delete(url(funding_category=funding_category))

    assert response.status_code == 204


def test_list_funding_categories(common_api_client1):
    FUNDING_CATEGORIES = 3
    FundingCategoryFactory.create_batch(size=FUNDING_CATEGORIES)

    response = common_api_client1.get(url())

    assert response.status_code == 200
    assert len(response.data) == FUNDING_CATEGORIES


def test_update_is_active(admin_api_client, funding_category):
    update = {
        "is_active": False,
    }

    response = admin_api_client.patch(url(funding_category=funding_category), update)

    assert response.status_code == 200
    assert response.data["is_active"] == update["is_active"]


def test_update_name(admin_api_client, funding_category):
    update = {
        "name": "NAME",
    }

    response = admin_api_client.patch(url(funding_category=funding_category), update)

    assert response.status_code == 200
    assert response.data["name"] == update["name"]


def test_update_slug(admin_api_client, funding_category):
    update = {
        "slug": "funding-category",
    }

    response = admin_api_client.patch(url(funding_category=funding_category), update)

    assert response.status_code == 200
    assert response.data["slug"] == update["slug"]
